package com.sf.service.gateway.task;

import cn.hutool.core.thread.ExecutorBuilder;
import cn.hutool.core.thread.RejectPolicy;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.sf.common.utils.URLUtils;
import com.sf.service.gateway.domain.GatewayRoute;
import com.sf.service.gateway.enums.GatewayDataStatus;
import com.sf.service.gateway.enums.RouteActiveStatus;
import com.sf.service.gateway.service.IGatewayRouteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 功能描述: 路由url活跃监测
 *
 * @author a_kun
 * @date 2024/5/6 10:11
 */
@Slf4j
@Component("gatewayRouteURLMonitor")
public class GatewayRouteURLMonitor {

    private static int CORE_POOL_SIZE = Runtime.getRuntime().availableProcessors() * 2;


    private static ThreadPoolExecutor threadPoolExecutor = ExecutorBuilder.create()
            .setCorePoolSize(CORE_POOL_SIZE).setMaxPoolSize(CORE_POOL_SIZE + 2)
            .setWorkQueue(new LinkedBlockingQueue<>(1024))
            .setThreadFactory(ThreadUtil.createThreadFactory("GatewayRouteURLMonitor"))
            .setHandler(RejectPolicy.DISCARD_OLDEST.getValue())
            .build();

    @Resource
    private IGatewayRouteService gatewayRouteService;

    public void monitor() {
        log.info("开始路由url活跃监测任务");
        GatewayRoute query = new GatewayRoute();
        query.setRouteStatusActiveMonitoring(GatewayDataStatus.ENABLE.getCode());
        List<GatewayRoute> routeList = gatewayRouteService.selectGatewayRouteList(query);
        routeList.forEach(route -> threadPoolExecutor.execute(() -> {
            log.info("路由规则：" + route.getRouteName() + "开始路由url活跃监测");
            if (StrUtil.isNotBlank(route.getRouteContent())) {
                boolean urlAlive = true;
                List<GatewayRoute.RouteContent> routeContentList = JSON.parseArray(route.getRouteContent(), GatewayRoute.RouteContent.class);
                for (GatewayRoute.RouteContent content : routeContentList) {
                    // 有一个异常就不用监测其他的了
                    if (!urlAlive) {
                        break;
                    }
                    String urlString = content.getServerAddress() + route.getRouteActiveMonitoringPath();
                    log.info("URL:" + urlString);
                    urlAlive = URLUtils.isUrlAlive(urlString, route.getRouteActiveMonitoringTimeout().intValue());
                }
                RouteActiveStatus routeActiveStatus = urlAlive ? RouteActiveStatus.NORMAL : RouteActiveStatus.ABNORMAL;
                GatewayRoute update = new GatewayRoute();
                update.setId(route.getId());
                update.setRouteActiveStatus(routeActiveStatus.getCode());
                gatewayRouteService.updateGatewayRoute(update);
            }
        }));
        log.info("结束路由url活跃监测任务");
    }


}
